import logging
import re
from datetime import datetime

from fastapi import APIRouter, Depends, HTTPException, status
from sqlalchemy import func
from sqlalchemy.orm import Session

from app.core.database import get_db
from app.dependencies.auth import get_current_admin
from app.models.tag import ProductTag, Tag
from app.models.user import User
from app.schemas.tag import (
    Tag as TagSchema,
)
from app.schemas.tag import (
    TagCreate,
    TagUpdate,
    TagWithProductCount,
)

router = APIRouter()
logger = logging.getLogger(__name__)

def slugify(text):
    """Convert a string to a URL-friendly slug."""
    # Remove non-alphanumeric characters
    text = re.sub(r'[^\w\s-]', '', text.lower())
    # Replace spaces with hyphens
    text = re.sub(r'[\s]+', '-', text)
    # Remove consecutive hyphens
    text = re.sub(r'[-]+', '-', text)
    # Add timestamp to ensure uniqueness
    timestamp = int(datetime.now().timestamp())
    return f"{text}-{timestamp}"

@router.get("/", response_model=list[TagSchema])
async def get_tags(
    skip: int = 0,
    limit: int = 100,
    db: Session = Depends(get_db)
):
    """
    Get all tags.
    """
    tags = db.query(Tag).offset(skip).limit(limit).all()
    return tags

@router.get("/with-count", response_model=list[TagWithProductCount])
async def get_tags_with_product_count(
    skip: int = 0,
    limit: int = 100,
    db: Session = Depends(get_db)
):
    """
    Get all tags with product counts.
    """
    tags = db.query(Tag).offset(skip).limit(limit).all()

    # Add product count to each tag
    for tag in tags:
        tag.product_count = db.query(ProductTag).filter(ProductTag.tag_id == tag.id).count()

    return tags

@router.get("/{tag_id}", response_model=TagSchema)
async def get_tag(
    tag_id: str,
    db: Session = Depends(get_db)
):
    """
    Get a specific tag by ID.
    """
    tag = db.query(Tag).filter(Tag.id == tag_id).first()
    if not tag:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Tag not found"
        )

    return tag

@router.post("/", response_model=TagSchema)
async def create_tag(
    tag_in: TagCreate,
    db: Session = Depends(get_db),
    current_user: User = Depends(get_current_admin)
):
    """
    Create a new tag (admin only).
    """
    # Check if name already exists
    existing_tag_name = db.query(Tag).filter(func.lower(Tag.name) == tag_in.name.lower()).first()
    if existing_tag_name:
        raise HTTPException(
            status_code=status.HTTP_400_BAD_REQUEST,
            detail="A tag with this name already exists"
        )

    # Check if slug already exists
    existing_tag_slug = db.query(Tag).filter(Tag.slug == tag_in.slug).first()
    if existing_tag_slug:
        # If slug exists, create a unique one
        tag_in.slug = slugify(tag_in.name)

    # Create new tag
    db_tag = Tag(
        name=tag_in.name,
        slug=tag_in.slug
    )

    db.add(db_tag)
    db.commit()
    db.refresh(db_tag)

    logger.info(f"Tag created: {db_tag.name} (ID: {db_tag.id})")
    return db_tag

@router.put("/{tag_id}", response_model=TagSchema)
async def update_tag(
    tag_id: str,
    tag_in: TagUpdate,
    db: Session = Depends(get_db),
    current_user: User = Depends(get_current_admin)
):
    """
    Update a tag (admin only).
    """
    tag = db.query(Tag).filter(Tag.id == tag_id).first()
    if not tag:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Tag not found"
        )

    # Check if name already exists (if updating name)
    if tag_in.name and tag_in.name.lower() != tag.name.lower():
        existing_tag = db.query(Tag).filter(func.lower(Tag.name) == tag_in.name.lower()).first()
        if existing_tag:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="A tag with this name already exists"
            )

    # Check if slug already exists (if updating slug)
    if tag_in.slug and tag_in.slug != tag.slug:
        existing_tag = db.query(Tag).filter(Tag.slug == tag_in.slug).first()
        if existing_tag:
            raise HTTPException(
                status_code=status.HTTP_400_BAD_REQUEST,
                detail="Slug already exists"
            )

    # Update tag attributes
    for key, value in tag_in.dict(exclude_unset=True).items():
        setattr(tag, key, value)

    db.commit()
    db.refresh(tag)

    logger.info(f"Tag updated: {tag.name} (ID: {tag.id})")
    return tag

@router.delete("/{tag_id}", response_model=dict)
async def delete_tag(
    tag_id: str,
    db: Session = Depends(get_db),
    current_user: User = Depends(get_current_admin)
):
    """
    Delete a tag (admin only).
    """
    tag = db.query(Tag).filter(Tag.id == tag_id).first()
    if not tag:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND,
            detail="Tag not found"
        )

    # Delete the tag
    db.delete(tag)
    db.commit()

    logger.info(f"Tag deleted: {tag.name} (ID: {tag.id})")
    return {"message": "Tag successfully deleted"}
